@@ -127,7 +127,8 @@ sub dcelify {
 	my ($src_pos) = $n2V->{$s}->pos();
 	foreach $t (keys %{ $n2E->{$s} }) {
 	    my ($tgt_pos) = $n2V->{$t}->pos();
-	    push @$neighbor, {name=>$t, angle=>($tgt_pos-$src_pos)->angle()};
+	    my ($a) = $tgt_pos - $src_pos;
+	    push @$neighbor, {name=>$t, angle=>atan2($a->[1], $a->[0])};
 	}
 	next unless $#$neighbor >= 0;
 	$neighbor = [ map { $_->{name} }
@@ -156,7 +157,7 @@ sub v_new {
     my ($self, $name, %opts) = @_;
     %opts = ( %{ $self->cget(-node_opts) }, %opts );
     # as always, the host should take care of prepending %opts with -node_opts
-    my ($pos) = Vector->new(delete $opts{-pos});
+    my ($pos) = Vector2->new(@{ delete $opts{-pos} });
     my ($v) = Vertex->new($self, $pos, -name=>$name, %opts);
 #    $v->configure(-text=>$v->cget(-display)->($v)) unless defined $opts{-text};
     return $v;
